@@ -823,7 +823,7 @@ static const ib_cfg_var_t cfg_vars_defaults[] = {
 	 STRUCT_FLD(validate,	ib_cfg_var_validate_numeric),
 	 STRUCT_FLD(set,	ib_cfg_var_set_generic),
 	 STRUCT_FLD(get,	ib_cfg_var_get_generic),
-	 STRUCT_FLD(tank,	&srv_log_buffer_size)},
+	 STRUCT_FLD(tank,	&srv_log_buffer_curr_size)},
 
 	{STRUCT_FLD(name,	"log_file_size"),
 	 STRUCT_FLD(type,	IB_CFG_ULINT),
@@ -833,7 +833,7 @@ static const ib_cfg_var_t cfg_vars_defaults[] = {
 	 STRUCT_FLD(validate,	ib_cfg_var_validate_numeric),
 	 STRUCT_FLD(set,	ib_cfg_var_set_generic),
 	 STRUCT_FLD(get,	ib_cfg_var_get_generic),
-	 STRUCT_FLD(tank,	&srv_log_file_size)},
+	 STRUCT_FLD(tank,	&srv_log_file_curr_size)},
 
 	{STRUCT_FLD(name,	"log_files_in_group"),
 	 STRUCT_FLD(type,	IB_CFG_ULINT),
