@@ -27,10 +27,32 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 extern "C" {
 #endif /* __cplusplus */
 
-/** InnoDB error codes. Most of the error codes are internal to the engine
-and will not be seen by user applications. The partial error codes reflect
-the sub-state of an operation within InnoDB. Some of the error codes are
-deprecated and are no longer used. */
+#if defined(BUILDING_HAILDB)
+# if defined(HAVE_VISIBILITY)
+#  define HAILDB_API __attribute__ ((visibility("default")))
+#  define HAILDB_LOCAL  __attribute__ ((visibility("hidden")))
+# elif defined (__SUNPRO_C) && (__SUNPRO_C >= 0x550)
+#  define HAILDB_API __global
+#  define HAILDB_LOCAL __hidden
+# elif defined(_MSC_VER)
+#  define HAILDB_API extern __declspec(dllexport)
+#  define HAILDB_LOCAL
+# endif /* defined(HAVE_VISIBILITY) */
+#else  /* defined(BUILDING_LIBDRIZZLE) */
+# if defined(_MSC_VER)
+#  define HAILDB_API extern __declspec(dllimport)
+#  define HAILDB_LOCAL
+# else
+#  define HAILDB_API
+#  define HAILDB_LOCAL
+# endif /* defined(_MSC_VER) */
+#endif /* defined(BUILDING_HAILDB) */
+
+/** \enum db_err InnoDB error codes.
+    Most of the error codes are internal to the engine
+    and will not be seen by user applications. The partial error codes reflect
+    the sub-state of an operation within InnoDB. Some of the error codes are
+    deprecated and are no longer used. */
 enum db_err {
 	DB_SUCCESS = 10,		/*!< A successult result */
 
@@ -200,6 +222,9 @@ enum db_err {
 #define strcasecmp		_stricmp
 #endif
 
+/** \def UNIV_NO_IGNORE
+    Some HailDB methods will produce warnings if the result of them is not
+    checked in client code. This uses a GCC compiler extension. */
 #if defined(__GNUC__) && (__GNUC__ > 2) && ! defined(__INTEL_COMPILER)
 #define UNIV_NO_IGNORE		__attribute__ ((warn_unused_result))
 #else
@@ -213,7 +238,9 @@ enum db_err {
 #define IB_FALSE		0x0UL
 
 /* Basic types used by the InnoDB API. */
-/** All InnoDB error codes are represented by ib_err_t */
+/** All InnoDB error codes are represented by ib_err_t. See \ref db_err for
+    a complete list of possible error codes.
+ */
 typedef enum db_err             ib_err_t;
 /** Representation of a byte within InnoDB */
 typedef unsigned char           ib_byte_t;
@@ -746,6 +773,7 @@ Add an UNSIGNED BIGINT NOT NULL column to a table schema.
 				8)
 /*! @def ib_cfg_set_int(name, value)
 Set an int configuration variable.
+@ingroup config
 @param name is the config variable name
 @param value is the integer value of the variable
 @return DB_SUCCESS or error code */
@@ -753,6 +781,7 @@ Set an int configuration variable.
 
 /*! @def ib_cfg_set_text(name, value)
 Set a text configuration variable.
+@ingroup config
 @param name is the config variable name
 @param value is the char* value of the variable
 @return DB_SUCCESS or error code */
@@ -760,18 +789,21 @@ Set a text configuration variable.
 
 /*! @def ib_cfg_set_bool_on(name)
 Set a boolean configuration variable to IB_TRUE.
+@ingroup config
 @param name is the config variable name
 @return DB_SUCCESS or error code */
 #define ib_cfg_set_bool_on(name)	ib_cfg_set(name, IB_TRUE)
 
 /*! @def ib_cfg_set_bool_off(name)
 Set a boolean configuration variable to IB_FALSE.
+@ingroup config
 @param name is the config variable name
 @return DB_SUCCESS or error code */
 #define ib_cfg_set_bool_off(name)	ib_cfg_set(name, IB_FALSE)
 
 /*! @def ib_cfg_set_callback(name, value)
 Set a generic ib_cb_t callback function.
+@ingroup config
 @param name is the config variable name
 @param value is a pointer to a callback function
 @return DB_SUCCESS or error code */
@@ -780,16 +812,17 @@ Set a generic ib_cb_t callback function.
 /** Callback function to compare InnoDB key columns in an index. */
 extern ib_client_cmp_t	ib_client_compare;
 
-/* Define the Doxygen groups:
+/** Define the Doxygen groups:
    @defgroup init Startup/Shutdown functions
    @defgroup cursor Cursor functions
    @defgroup trx Transaction functions
-   @defgroup conf Configuration functions
+   @defgroup config Configuration functions
    @defgroup ddl DDL functions
    @defgroup misc Miscellaneous functions
    @defgroup tuple Tuple functions
    @defgroup sql SQL functions
    @defgroup dml DML functions
+   @defgroup debug Debug and Testing functions
 */
 /*****************************************************************//**
 Return the API version number, the version number format is:
@@ -807,6 +840,7 @@ Return the API version number, the version number format is:
 @ingroup misc
 @return	API version number */
 
+HAILDB_API
 ib_u64_t
 ib_api_version(void) UNIV_NO_IGNORE;
 /*=================*/
@@ -820,6 +854,7 @@ functions should be called.
 @ingroup init
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_init(void) UNIV_NO_IGNORE;
 /*=========*/
@@ -832,10 +867,11 @@ shutdown cleanly but the user deleted the REDO log files then it
 will recreate the REDO log files.
 
 @ingroup init
-@param format is the max file format name that the engine supports
+@param format is the max file format name that the engine supports. Currently this is either Antelope or Barracuda although more may be added in the future without API changes.
 @return	DB_SUCCESS or error code
 @see DB_SUCCESS */
 
+HAILDB_API
 ib_err_t
 ib_startup(
 /*=======*/
@@ -851,6 +887,7 @@ default values.
 @param flag is the shutdown flag
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_shutdown(
 /*========*/
@@ -869,6 +906,7 @@ by calling ib_trx_release().
 @param ib_trx_level is the transaction isolation level
 @return	innobase txn handle */
 
+HAILDB_API
 ib_err_t
 ib_trx_start(
 /*=========*/
@@ -883,6 +921,7 @@ put the transaction in the active state.
 @param ib_trx_level is the transaction isolation level
 @return	innobase txn handle */
 
+HAILDB_API
 ib_trx_t
 ib_trx_begin(
 /*=========*/
@@ -895,12 +934,15 @@ in the trx_is_interrupted callback. HailDB will only ever pass this
 around, it will never dereference it.
 
 @ingroup trx
+@param ib_trx is the transaction to set the client data for
 @param client_data is client program's data about this transaction */
+
+HAILDB_API
 void
 ib_trx_set_client_data(
 /*=========*/
 	ib_trx_t	ib_trx,		/*!< in: transaction */
-	void*		client_thd);
+	void*		client_data);
 
 /*****************************************************************//**
 Query the transaction's state. This function can be used to check for
@@ -914,6 +956,7 @@ code indicating this. @see DB_DEADLOCK, @see DB_LOCK_TABLE_FULL and
 @param ib_trx is the transaction handle
 @return	transaction state */
 
+HAILDB_API
 ib_trx_state_t
 ib_trx_state(
 /*=========*/
@@ -928,6 +971,7 @@ to free the transaction handle.
 @param ib_trx is the transaction handle
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_trx_release(
 /*===========*/
@@ -941,6 +985,7 @@ It will also free the transaction handle.
 @param ib_trx is thr transaction handle
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_trx_commit(
 /*==========*/
@@ -954,6 +999,7 @@ It will also free the transaction handle.
 @param ib_trx is the transaction handle
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_trx_rollback(
 /*============*/
@@ -973,6 +1019,7 @@ add the column definitions to the table schema.
 @param len is the maximum length of the column
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_table_schema_add_col(
 /*====================*/
@@ -994,6 +1041,7 @@ the table schema instance is freed.
 @param[out] ib_idx_sch is the key definition schema instance
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_table_schema_add_index(
 /*======================*/
@@ -1007,6 +1055,7 @@ Destroy a schema. The handle is freed by this function.
 @ingroup ddl
 @param ib_tbl_sch is the table schema to delte*/
 
+HAILDB_API
 void
 ib_table_schema_delete(
 /*===================*/
@@ -1023,6 +1072,7 @@ Create a table schema.
 
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_table_schema_create(
 /*===================*/
@@ -1040,6 +1090,7 @@ Add columns to an index schema definition.
 @param prefix_len is the prefix length of the index or 0 if no prefix
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_index_schema_add_col(
 /*====================*/
@@ -1057,6 +1108,7 @@ Create an index schema instance.
 @param[out] ib_idx_sch is the newly created index schema instance
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_index_schema_create(
 /*===================*/
@@ -1072,6 +1124,7 @@ Set index as clustered index. Implies UNIQUE.
 @param[in,out] ib_idx_sch is the index schema to update
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_index_schema_set_clustered(
 /*==========================*/
@@ -1083,6 +1136,7 @@ Set to true if it's a simple select.
 @ingroup sql
 @param[in, out] ib_crsr is the cursor to update */
 
+HAILDB_API
 void
 ib_cursor_set_simple_select(
 /*========================*/
@@ -1095,6 +1149,7 @@ Set index as a unique index.
 @param[in,out] ib_idx_sch is the index schema to update
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_index_schema_set_unique(
 /*=======================*/
@@ -1106,6 +1161,7 @@ Destroy an index schema.
 @ingroup ddl
 @param ib_idx_sch is the index schema to delete */
 
+HAILDB_API
 void
 ib_index_schema_delete(
 /*===================*/
@@ -1122,6 +1178,7 @@ DB_TABLE_IS_BEING_USED and id will contain that table's id.
 @param[out] id table id that was created
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_table_create(
 /*============*/
@@ -1139,6 +1196,7 @@ exclusive mode.
 @param new_name the new name for the table
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_table_rename(
 /*============*/
@@ -1155,6 +1213,7 @@ Create a secondary index. The index id encodes the table id in the high
 @param[out] index_id is the new index id that was created
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_index_create(
 /*============*/
@@ -1170,6 +1229,7 @@ exclusive mode.
 @param name is the name of the table to drop
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_table_drop(
 /*==========*/
@@ -1185,6 +1245,7 @@ exclusive mode.
 @param index_id is the id of the index to drop
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_index_drop(
 /*==========*/
@@ -1200,6 +1261,7 @@ Open an InnoDB table and return a cursor handle to it.
 @param[out] ib_crsr is the new cursor
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_open_table_using_id(
 /*==========================*/
@@ -1216,6 +1278,7 @@ Open an InnoDB index and return a cursor handle to it.
 @param[out] ib_crsr is the new cursor
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_open_index_using_id(
 /*==========================*/
@@ -1232,6 +1295,7 @@ Open an InnoDB secondary index cursor and return a cursor handle to it.
 @param[out] ib_crsr is the new cursor
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_open_index_using_name(
 /*============================*/
@@ -1248,6 +1312,7 @@ Open an InnoDB table by name and return a cursor handle to it.
 @param ib_crsr is the new cursor
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_open_table(
 /*=================*/
@@ -1262,6 +1327,7 @@ Reset the cursor.
 @param ib_crsr is an open cursor
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_reset(
 /*============*/
@@ -1274,6 +1340,7 @@ Close an InnoDB table and free the cursor.
 @param ib_crsr is an open cursor
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_close(
 /*============*/
@@ -1287,6 +1354,7 @@ Insert a row to a table.
 @param ib_tpl is the tuple to insert
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_insert_row(
 /*=================*/
@@ -1302,6 +1370,7 @@ Update a row in a table.
 @param ib_new_tpl is the new tuple with the updated values
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_update_row(
 /*=================*/
@@ -1316,6 +1385,7 @@ Delete a row in a table.
 @param ib_crsr is the cursor instance
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_delete_row(
 /*=================*/
@@ -1329,6 +1399,7 @@ Read current row.
 @param[out] ib_tpl is the tuple to read the column values
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_read_row(
 /*===============*/
@@ -1342,6 +1413,7 @@ Move cursor to the prev user record in the table.
 @param ib_crsr is the cursor instance
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_prev(
 /*===========*/
@@ -1354,6 +1426,7 @@ Move cursor to the next user record in the table.
 @param ib_crsr is the cursor instance
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_next(
 /*===========*/
@@ -1366,6 +1439,7 @@ Move cursor to the first record in the table.
 @param ib_crsr is the cursor instance
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_first(
 /*============*/
@@ -1378,6 +1452,7 @@ Move cursor to the last record in the table.
 @param ib_crsr is the cursor instance
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_last(
 /*===========*/
@@ -1394,6 +1469,7 @@ Search for key.
        the current row
 @return	DB_SUCCESS or err code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_moveto(
 /*=============*/
@@ -1410,6 +1486,7 @@ attached to another transaction.
 @param ib_crsr is the cursor instance
 @param ib_trx is the transaction to attach to the cursor */
 
+HAILDB_API
 void
 ib_cursor_attach_trx(
 /*=================*/
@@ -1422,6 +1499,7 @@ Set the client comparison function for BLOBs and client types.
 @ingroup misc
 @param client_cmp_func is the index key compare callback function */
 
+HAILDB_API
 void
 ib_set_client_compare(
 /*==================*/
@@ -1434,6 +1512,7 @@ Set the match mode for ib_cursor_move().
 @param ib_crsr is the cursor instance
 @param match_mode is the match mode to set */
 
+HAILDB_API
 void
 ib_cursor_set_match_mode(
 /*=====================*/
@@ -1450,6 +1529,7 @@ Set a column of the tuple. Make a copy using the tuple's heap.
 @param len is the data value (src) length in bytes
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_col_set_value(
 /*=============*/
@@ -1466,6 +1546,7 @@ Get the size of the data available in the column the tuple.
 @param i is the index (ordinal position) of the column within the tuple
 @return	bytes avail or IB_SQL_NULL */
 
+HAILDB_API
 ib_ulint_t
 ib_col_get_len(
 /*===========*/
@@ -1482,6 +1563,7 @@ Copy a column value from the tuple.
 @param len is the maximum number of bytes that can be copied to dst
 @return	bytes copied or IB_SQL_NULL */
 
+HAILDB_API
 ib_ulint_t
 ib_col_copy_value(
 /*==============*/
@@ -1499,6 +1581,7 @@ Read a signed int 8 bit column from an InnoDB tuple.
 @param[out] ival is the integer value
 @return	DB_SUCCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_read_i8(
 /*=============*/
@@ -1515,6 +1598,7 @@ Read an unsigned int 8 bit column from an InnoDB tuple.
 @param[out] ival is the integer value
 @return	DB_SUCCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_read_u8(
 /*=============*/
@@ -1531,6 +1615,7 @@ Read a signed int 16 bit column from an InnoDB tuple.
 @param[out] ival is the integer value
 @return	DB_SUCCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_read_i16(
 /*==============*/
@@ -1547,6 +1632,7 @@ Read an unsigned int 16 bit column from an InnoDB tuple.
 @param[out] ival is the integer value
 @return	DB_SUCCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_read_u16(
 /*==============*/
@@ -1563,6 +1649,7 @@ Read a signed int 32 bit column from an InnoDB tuple.
 @param[out] ival is the integer value
 @return	DB_SUCCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_read_i32(
 /*==============*/
@@ -1579,6 +1666,7 @@ Read an unsigned int 32 bit column from an InnoDB tuple.
 @param[out] ival is the integer value
 @return	DB_SUCCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_read_u32(
 /*==============*/
@@ -1595,6 +1683,7 @@ Read a signed int 64 bit column from an InnoDB tuple.
 @param[out] ival is the integer value
 @return	DB_SUCCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_read_i64(
 /*==============*/
@@ -1611,6 +1700,7 @@ Read an unsigned int 64 bit column from an InnoDB tuple.
 @param[out] ival is the integer value
 @return	DB_SUCCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_read_u64(
 /*==============*/
@@ -1626,6 +1716,7 @@ Get a column value pointer from the tuple.
 @param i is the index (ordinal position) of the column within the tuple
 @return	NULL or pointer to buffer */
 
+HAILDB_API
 const void*
 ib_col_get_value(
 /*=============*/
@@ -1641,6 +1732,7 @@ Get a column type, length and attributes from the tuple.
 @param[out] ib_col_meta the column meta data
 @return	len of column data */
 
+HAILDB_API
 ib_ulint_t
 ib_col_get_meta(
 /*============*/
@@ -1655,6 +1747,7 @@ ib_col_get_meta(
 @param ib_tpl is the tuple to be freed
 @return	new tuple, or NULL */
 
+HAILDB_API
 ib_tpl_t
 ib_tuple_clear(
 /*============*/
@@ -1671,6 +1764,7 @@ to the cluster key. It does a deep copy of the column data.
 @param ib_src_tpl is the source secondary index tuple to copy from
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_tuple_get_cluster_key(
 /*=====================*/
@@ -1687,6 +1781,7 @@ must be of the same type and belong to the same table/index.
 @param ib_src_tpl is the source tuple
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_tuple_copy(
 /*==========*/
@@ -1700,6 +1795,7 @@ Create an InnoDB tuple used for index/table search.
 @param ib_crsr is the cursor instance
 @return tuple for current index */
 
+HAILDB_API
 ib_tpl_t
 ib_sec_search_tuple_create(
 /*=======================*/
@@ -1712,6 +1808,7 @@ Create an InnoDB tuple used for index/table search.
 @param ib_crsr is the cursor instance
 @return	tuple for current index */
 
+HAILDB_API
 ib_tpl_t
 ib_sec_read_tuple_create(
 /*=====================*/
@@ -1724,6 +1821,7 @@ Create an InnoDB tuple used for table key operations.
 @param ib_crsr is the cursor instance
 @return	tuple for current table */
 
+HAILDB_API
 ib_tpl_t
 ib_clust_search_tuple_create(
 /*=========================*/
@@ -1736,6 +1834,7 @@ Create an InnoDB tuple for table row operations.
 @param ib_crsr is the cursor instance
 @return	tTuple for current table */
 
+HAILDB_API
 ib_tpl_t
 ib_clust_read_tuple_create(
 /*=======================*/
@@ -1748,6 +1847,7 @@ Return the number of user columns in the tuple definition.
 @param ib_tpl is a tuple
 @return	number of user columns */
 
+HAILDB_API
 ib_ulint_t
 ib_tuple_get_n_user_cols(
 /*=====================*/
@@ -1760,6 +1860,7 @@ Return the number of columns in the tuple definition.
 @param ib_tpl is a tuple
 @return	number of columns */
 
+HAILDB_API
 ib_ulint_t
 ib_tuple_get_n_cols(
 /*================*/
@@ -1771,6 +1872,7 @@ Destroy an InnoDB tuple.
 @ingroup tuple
 @param ib_tpl is the tuple instance to delete */
 
+HAILDB_API
 void
 ib_tuple_delete(
 /*============*/
@@ -1785,6 +1887,7 @@ on success.
 @param[out] table_id is the new table id
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_truncate(
 /*===============*/
@@ -1799,6 +1902,7 @@ Truncate a table.
 @param[out] table_id is the new table id
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_table_truncate(
 /*==============*/
@@ -1813,6 +1917,7 @@ Get a table id.
 @param[out] table_id is the new table id if found
 @return	DB_SUCCESS if found */
 
+HAILDB_API
 ib_err_t
 ib_table_get_id(
 /*============*/
@@ -1828,6 +1933,7 @@ Get an index id.
 @param[out] index_id contains the index id if found
 @return	DB_SUCCESS if found */
 
+HAILDB_API
 ib_err_t
 ib_index_get_id(
 /*============*/
@@ -1842,6 +1948,7 @@ Create a database if it doesn't exist.
 @param dbname is the name of the database to create
 @return	IB_TRUE on success */
 
+HAILDB_API
 ib_bool_t
 ib_database_create(
 /*===============*/
@@ -1855,6 +1962,7 @@ within the database.
 @param dbname is the name of the database to drop
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_database_drop(
 /*=============*/
@@ -1867,6 +1975,7 @@ Check if cursor is positioned.
 @param ib_crsr is the cursor instance to check
 @return	IB_TRUE if positioned */
 
+HAILDB_API
 ib_bool_t
 ib_cursor_is_positioned(
 /*====================*/
@@ -1879,6 +1988,7 @@ Latches the data dictionary in shared mode.
 @param ib_trx is the transaction instance
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_schema_lock_shared(
 /*==================*/
@@ -1891,6 +2001,7 @@ Latches the data dictionary in exclusive mode.
 @param ib_trx is the transaction instance
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_schema_lock_exclusive(
 /*======================*/
@@ -1904,6 +2015,7 @@ user transaction.
 @param ib_trx is a transaction instance
 @return	TRUE if exclusive latch */
 
+HAILDB_API
 ib_bool_t
 ib_schema_lock_is_exclusive(
 /*========================*/
@@ -1914,6 +2026,7 @@ Checks if the data dictionary is latched in shared mode.
 @param ib_trx is a transaction instance
 @return	TRUE if shared latch */
 
+HAILDB_API
 ib_bool_t
 ib_schema_lock_is_shared(
 /*=====================*/
@@ -1926,6 +2039,7 @@ Unlocks the data dictionary.
 @param ib_trx is a transaction instance
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_schema_unlock(
 /*=============*/
@@ -1939,6 +2053,7 @@ Lock an InnoDB cursor/table.
 @param ib_lck_mode is the lock mode
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_lock(
 /*===========*/
@@ -1954,6 +2069,7 @@ Set the Lock an InnoDB table using the table id.
 @param ib_lck_mode is the lock mode
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_table_lock(
 /*===========*/
@@ -1969,6 +2085,7 @@ Set the Lock mode of the cursor.
 @param  ib_lck_mode is the lock mode
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_cursor_set_lock_mode(
 /*====================*/
@@ -1981,6 +2098,7 @@ Set need to access clustered index record flag.
 @ingroup dml
 @param ib_crsr is the cursor instance for which we want to set the flag */
 
+HAILDB_API
 void
 ib_cursor_set_cluster_access(
 /*=========================*/
@@ -2005,6 +2123,7 @@ The caller must have an exclusive lock on the InnoDB data dictionary
 @param arg is the argument passed to the visitor functions.
 @return	DB_SUCCESS or DB_ERROR */
 
+HAILDB_API
 ib_err_t
 ib_table_schema_visit(
 /*==================*/
@@ -2029,6 +2148,7 @@ The function will abort if visitor.tables() returns non-zero.
 @param arg argument passed to the visitor function
 @return DB_SUCCESS if successful */
 
+HAILDB_API
 ib_err_t
 ib_schema_tables_iterate(
 /*=====================*/
@@ -2046,6 +2166,7 @@ variable with name "name" was found and "type" was set.
 @param[out] type is the type of the variable name if found
 @return	DB_SUCCESS if successful */
 
+HAILDB_API
 ib_err_t
 ib_cfg_var_get_type(
 /*================*/
@@ -2061,6 +2182,7 @@ variable with name "name" was found and if its value was set.
 @param name is the config variable name whose value is to be set
 @return	DB_SUCCESS if set */
 
+HAILDB_API
 ib_err_t
 ib_cfg_set(
 /*=======*/
@@ -2077,6 +2199,7 @@ if the variable with name "name" was found and "value" was set.
 @param[out] value is the value of the variable if found
 @return	DB_SUCCESS if retrieved successfully */
 
+HAILDB_API
 ib_err_t
 ib_cfg_get(
 /*=======*/
@@ -2093,6 +2216,7 @@ when it is not needed anymore and for not modifying the individual strings.
 @param[out] names_num number of strings returned
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_cfg_get_all(
 /*===========*/
@@ -2110,6 +2234,7 @@ commit or rollback.
 @param name is the name of the savepoint
 @param name_len is the length of name in bytes */
 
+HAILDB_API
 void
 ib_savepoint_take(
 /*==============*/
@@ -2128,6 +2253,7 @@ savepoint are left as is.
 @return	if no savepoint of the name found then DB_NO_SAVEPOINT,
         otherwise DB_SUCCESS */
 
+HAILDB_API
 ib_err_t
 ib_savepoint_release(
 /*=================*/
@@ -2151,6 +2277,7 @@ savepoints are rolled back.
 @return	if no savepoint of the name found then DB_NO_SAVEPOINT,
         otherwise DB_SUCCESS */
 
+HAILDB_API
 ib_err_t
 ib_savepoint_rollback(
 /*==================*/
@@ -2168,6 +2295,7 @@ format and will need to be converted from the host format.
 @param val is the value to write
 @return	DB_SUCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_write_i8(
 /*==============*/
@@ -2185,6 +2313,7 @@ format and will need to be converted from the host format.
 @param val is the value to write
 @return	DB_SUCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_write_i16(
 /*=================*/
@@ -2202,6 +2331,7 @@ format and will need to be converted from the host format.
 @param val is the value to write
 @return	DB_SUCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_write_i32(
 /*===============*/
@@ -2219,6 +2349,7 @@ format and will need to be converted from the host format.
 @param val is the value to write
 @return	DB_SUCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_write_i64(
 /*===============*/
@@ -2236,6 +2367,7 @@ format and will need to be converted from the host format.
 @param val is the value to write
 @return	DB_SUCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_write_u8(
 /*==============*/
@@ -2253,6 +2385,7 @@ format and will need to be converted from the host format.
 @param val is the value to write
 @return	DB_SUCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_write_u16(
 /*===============*/
@@ -2270,6 +2403,7 @@ format and will need to be converted from the host format.
 @param val is the value to write
 @return	DB_SUCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_write_u32(
 /*=================*/
@@ -2287,6 +2421,7 @@ format and will need to be converted from the host format.
 @param val is the value to write
 @return	DB_SUCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_write_u64(
 /*===============*/
@@ -2300,6 +2435,7 @@ Inform the cursor that it's the start of an SQL statement.
 @ingroup cursor
 @param ib_crsr is the cursor instance */
 
+HAILDB_API
 void
 ib_cursor_stmt_begin(
 /*=================*/
@@ -2314,6 +2450,7 @@ Write a double value to a column.
 @param val is the value to write
 @return	DB_SUCCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_write_double(
 /*==================*/
@@ -2330,6 +2467,7 @@ Read a double column value from an InnoDB tuple.
 @param[out] dval is where the value is copied
 @return	DB_SUCCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_read_double(
 /*=================*/
@@ -2346,6 +2484,7 @@ Write a float value to a column.
 @param val is the value to write
 @return	DB_SUCCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_write_float(
 /*=================*/
@@ -2362,6 +2501,7 @@ Read a float value from an InnoDB tuple.
 @param[out] fval is where the value is copied
 @return	DB_SUCCESS or error */
 
+HAILDB_API
 ib_err_t
 ib_tuple_read_float(
 /*================*/
@@ -2377,6 +2517,7 @@ Set the message logging function.
 @param ib_msg_stream is the message stream, this is the first argument
        to the loggingfunction */
 
+HAILDB_API
 void
 ib_logger_set(
 /*==========*/
@@ -2391,6 +2532,7 @@ returned string is static and should not be freed or modified.
 @param db_errno is the error number
 @return	string, describing the error */
 
+HAILDB_API
 const char*
 ib_strerror(
 /*========*/
@@ -2406,6 +2548,7 @@ Get the value of an INT status variable.
 	DB_DATA_MISMATCH if found but type is not INT,
 	DB_NOT_FOUND otherwise. */
 
+HAILDB_API
 ib_err_t
 ib_status_get_i64(
 /*==============*/
@@ -2422,22 +2565,72 @@ when it is not needed anymore and for not modifying the individual strings.
 @param[out] names_num number of strings returned
 @return	DB_SUCCESS or error code */
 
+HAILDB_API
 ib_err_t
 ib_status_get_all(
 /*===========*/
 	const char***	names,
 	ib_u32_t*	names_num) UNIV_NO_IGNORE;
 
+/**
+   Type of callback in the event of HailDB panicing. Your callback should
+   call exit() rather soon, as continuing after a panic will lead to errors
+   returned from every API function. We have also not fully tested
+   every possible outcome from not immediately calling exit().
+*/
 typedef void (*ib_panic_handler_t)(void*, int, char*, ...);
 
+/**
+   Set panic handler.
+
+   HailDB will "panic" upon finding certain forms of corruption.
+   By setting a panic handler, you can implement your own notification
+   to the end user of this corruption (e.g. popping up a dialog box).
+
+   @ingroup config
+   @param new_panic_handler your panic handler
+*/
+HAILDB_API
 void
 ib_set_panic_handler(ib_panic_handler_t new_panic_handler);
 
+/**
+   Callback for checking if a transaction has been interrupted.
+   This callback lets you implement the MySQL KILL command kind of
+   functionality.
+   A transaction may block in the thread it's running in (for example, while
+   acquiring row locks or doing IO) but other threads may do something that
+   causes ib_trx_is_interrupted_handler_t to return true.
+*/
 typedef int (*ib_trx_is_interrupted_handler_t)(void*);
 
+/**
+   Set trx_is_interrupted_handler.
+
+   You may specificy a callback that HailDB will check during certain wait
+   situations to see if it should abort the operation or not. This lets
+   you implement MySQL/Drizzle KILL command style functionality.
+
+   @ingroup config
+   @param handler the trx_is_interrupted callback
+*/
+HAILDB_API
 void
 ib_set_trx_is_interrupted_handler(ib_trx_is_interrupted_handler_t handler);
 
+/**
+   Get which key caused a duplicate key error.
+
+   In the event of \ref DB_DUPLICATE_KEY error, you can call this function
+   immediately after to get the name of the table and index that caused
+   the error.
+
+   @ingroup trx
+   @param ib_trx Transaction where error occured
+   @param table_name pointer to be set to table_name. Valid until next ib_ function call. If you would like to keep it, make a copy.
+   @param index_name pointer to be set to the index name. Valid until next ib_ function call. If you would like to keep it, make a copy.
+*/
+HAILDB_API
 ib_err_t
 ib_get_duplicate_key(ib_trx_t ib_trx, const char **table_name, const char **index_name);
 
@@ -2464,9 +2657,66 @@ typedef struct {
 			       	this is only used for heuristics */
 } ib_table_stats_t;
 
+
+/** Get table statistics.
+
+   This function will fill out the provided \ref ib_table_stats_t with
+   statistics about the table on the currently opened \ref ib_crsr_t
+
+   @ingroup misc
+   @param ib_crsr A Cursor that is opened to a table
+   @param table_stats a \ref ib_table_stats_t to be filled out by HailDB
+   @param sizeof_ib_table_stats_t sizeof(ib_table_stats_t). This allows for ABI compatible changes to the size of ib_table_stats_t.
+   @returns \ref DB_SUCCESS or error
+*/
+HAILDB_API
 ib_err_t
 ib_get_table_statistics(ib_crsr_t ib_crsr, ib_table_stats_t *table_stats, size_t sizeof_ib_table_stats_t);
 
+/** Get statistics on number of different key values per index part
+
+    This function returns the approximate different key
+    values for this index. They are periodically recalculated.
+
+    @ingroup misc
+    @param ib_crsr A Cursor that is opened to a table
+    @param index_name name of the index
+    @param ncols returns the number of elements in n_diff
+    @param n_diff An array allocated with malloc() (user needs to free()) containing the statistics
+    @returns \ref DB_SUCCESS or error. \ref DB_NOT_FOUND if index is not found
+ */
+HAILDB_API
+ib_err_t
+ib_get_index_stat_n_diff_key_vals(ib_crsr_t ib_crsr, const char* index_name, ib_u64_t *ncols, ib_i64_t **n_diff);
+
+/** Force an update of table and index statistics
+
+    This function forces an update to the table and index statistics for
+    the table crsr is opened on.
+
+    @ingroup misc
+    @param crsr A Cursor that is opened to a table
+    @returns \ref DB_SUCCESS or error.
+ */
+HAILDB_API
+ib_err_t
+ib_update_table_statistics(ib_crsr_t crsr);
+
+/** Inject an error into HailDB
+
+    This function will simulate an error condition inside HailDB.
+    You should not rely on this function. It is for HailDB test suite use only,
+    parts may only be compiled into debug libraries and this function
+    can quite legitimately just return DB_ERROR and cause Voldemort to pay
+    you a visit.
+
+   @ingroup debug
+   @param error_to_inject The error inject code to insert.
+*/
+HAILDB_API
+ib_err_t
+ib_error_inject(int error_to_inject);
+
 #ifdef __cplusplus
 } /* extern "C" */
 #endif /* __cplusplus */
