@@ -101,7 +101,9 @@ extern ibool	srv_created_new_raw;
 
 extern ulint	srv_n_log_files;
 extern ulint	srv_log_file_size;
+extern ulint	srv_log_file_curr_size;
 extern ulint	srv_log_buffer_size;
+extern ulint	srv_log_buffer_curr_size;
 extern ulong	srv_flush_log_at_trx_commit;
 extern ibool	srv_adaptive_flushing;
 
@@ -581,6 +583,6 @@ extern ulint	srv_n_threads_active[];
 
 extern void* ib_panic_data;
 typedef void (*ib_panic_function_t)(void*, int, char*, ...);
-void srv_panic(int panic_ib_error, char* fmt, ...);
+UNIV_INTERN void srv_panic(int panic_ib_error, char* fmt, ...);
 
 #endif
