@@ -163,8 +163,10 @@ UNIV_INTERN ibool	srv_created_new_raw	= FALSE;
 UNIV_INTERN ulint	srv_n_log_files		= ULINT_MAX;
 /** Size in database pages */
 UNIV_INTERN ulint	srv_log_file_size	= ULINT_MAX;
+UNIV_INTERN ulint	srv_log_file_curr_size	= ULINT_MAX;
 /** Size in database pages */
 UNIV_INTERN ulint	srv_log_buffer_size	= ULINT_MAX;
+UNIV_INTERN ulint	srv_log_buffer_curr_size = ULINT_MAX;
 UNIV_INTERN ulong	srv_flush_log_at_trx_commit = 1;
 
 /** Try to flush dirty pages so as to avoid IO bursts at
@@ -693,7 +695,9 @@ srv_var_init(void)
 
 	srv_n_log_files = ULINT_MAX;
 	srv_log_file_size = ULINT_MAX;
+	srv_log_file_curr_size = ULINT_MAX;
 	srv_log_buffer_size = ULINT_MAX;
+	srv_log_buffer_curr_size = ULINT_MAX;
 	srv_flush_log_at_trx_commit = 1;
 
 	srv_buf_pool_size = ULINT_MAX;
@@ -1138,9 +1142,11 @@ srv_normalize_init_values(void)
 	srv_last_file_size_max = srv_last_file_size_max
 		* ((1024 * 1024) / UNIV_PAGE_SIZE);
 
-	srv_log_file_size = srv_log_file_size / UNIV_PAGE_SIZE;
+	srv_log_file_size = srv_log_file_curr_size / UNIV_PAGE_SIZE;
+	srv_log_file_curr_size = srv_log_file_size * UNIV_PAGE_SIZE;
 
-	srv_log_buffer_size = srv_log_buffer_size / UNIV_PAGE_SIZE;
+	srv_log_buffer_size = srv_log_buffer_curr_size / UNIV_PAGE_SIZE;
+	srv_log_buffer_curr_size = srv_log_buffer_size * UNIV_PAGE_SIZE;
 
 	srv_lock_table_size = 5 * (srv_buf_pool_size / UNIV_PAGE_SIZE);
 
