@@ -265,7 +265,10 @@ print_tuple(
 			case IB_CHAR:
 			case IB_BLOB:
 			case IB_DECIMAL:
-			case IB_VARCHAR: {
+			case IB_VARCHAR:
+			case IB_VARCHAR_ANYCHARSET:
+			case IB_CHAR_ANYCHARSET:
+			{
 				const char*	ptr;
 
 				ptr = ib_col_get_value(tpl, i);
